#!/usr/bin/env python2
"""
Define the constants for hostapd_binder
"""

import os

# hostapd configuration files
CHANNEL = 6
SSID = 'test'
KARMA_ENABLE = 0
BEACON_INT = 100
HW_MODE = 'g'
WPA_PASSPHRASE = ''
INTERFACE = 'wlan0'
VALID_HW_MODES = ['a', 'b', 'g']

# hostapd command line options
HOSTAPD_DEBUG_OFF = 0
HOSTAPD_DEBUG_ON = 1
HOSTAPD_DEBUG_VERBOSE = 2

DN = open(os.devnull, 'w')
HOSTAPD_DIR = 'hostapd-2_6/hostapd'
HOSTAPD_SHARED_LIB_PATH = os.path.join(HOSTAPD_DIR, 'libhostapd.so')
HOSTAPD_EXE_PATH = os.path.join(HOSTAPD_DIR, 'hostapd')
HOSTAPD_CONF_PATH = '/tmp/hostapd.conf'
DENY_MACS_PATH = '/tmp/hostapd.deny'

# Console colors

WHITE = '\033[0m'  # white (normal)
RED = '\033[31m'   # red
TAN = '\033[93m'   # tan

# build related
MAKE_CMD = ["make", "hostapd_lib"]
CP_CMD = ['cp', 'defconfig', '.config']



"""
Module for defining the required hostapd macros and source files
"""
import os


# define directory macros
HOSTAPD_DIR = 'roguehostapd/hostapd-2_6'
HOSTAPD_SRC = os.path.join(HOSTAPD_DIR, 'src')
HOSTAPD_MAIN = os.path.join(HOSTAPD_DIR, 'hostapd')
HOSTAPD_AP = os.path.join(HOSTAPD_SRC, 'ap')
HOSTAPD_UTILS = os.path.join(HOSTAPD_SRC, 'utils')
HOSTAPD_WPS = os.path.join(HOSTAPD_SRC, 'wps')
HOSTAPD_DRVS = os.path.join(HOSTAPD_SRC, 'drivers')
HOSTAPD_COMMON = os.path.join(HOSTAPD_SRC, 'common')
HOSTAPD_L2_PACKET = os.path.join(HOSTAPD_SRC, 'l2_packet')
HOSTAPD_EAPOL_AUTH = os.path.join(HOSTAPD_SRC, 'eapol_auth')
HOSTAPD_RADIUS = os.path.join(HOSTAPD_SRC, 'radius')
HOSTAPD_EAP_SERVER = os.path.join(HOSTAPD_SRC, 'eap_server')
HOSTAPD_EAP_COMMON = os.path.join(HOSTAPD_SRC, 'eap_common')
HOSTAPD_CRYPTO = os.path.join(HOSTAPD_SRC, 'crypto')
SHARED_LIB_PATH = os.path.join(HOSTAPD_MAIN, 'libhostapd')
LIB_NL3_PATH = '/usr/include/libnl3'
LIB_SSL_PATH = '/usr/include/openssl'

# files under hostapd-2.6/hostapd
MAIN_FILES = ['main.c', 'config_file.c', 'eap_register.c', 'ctrl_iface.c']
# files under hostapd-2.6/src/l2_packet
L2_PACKET_FILES = ['l2_packet_linux.c']
# files under hostapd-2.6/src/ap
AP_FILES = ['hostapd.c',
            'wpa_auth_glue.c',
            'ap_list.c',
            'dfs.c',
            'ap_drv_ops.c',
            'utils.c',
            'authsrv.c',
            'ieee802_1x.c',
            'ap_config.c',
            'eap_user_db.c',
            'ieee802_11_auth.c',
            'ieee802_11.c',
            'sta_info.c',
            'wpa_auth.c',
            'tkip_countermeasures.c',
            'ap_mlme.c',
            'wpa_auth_ie.c',
            'preauth_auth.c',
            'pmksa_cache_auth.c',
            'ieee802_11_shared.c',
            'beacon.c',
            'accounting.c',
            'bss_load.c',
            'rrm.c',
            'drv_callbacks.c',
            'neighbor_db.c',
            'ieee802_11_ht.c',
            'ctrl_iface_ap.c',
            'hw_features.c',
            'wmm.c',
            'karma_handlers.c',
            'wps_hostapd.c']
# files under hostapd-2.6/src/common
COMMON_FILES = ['hw_features_common.c',
                'ctrl_iface_common.c',
                'ieee802_11_common.c',
                'wpa_common.c']
# files under hostapd-2.6/src/drivers
DRV_FILES = ['drivers.c',
             'driver_common.c',
             'driver_wired.c',
             'driver_nl80211.c',
             'driver_nl80211_capa.c',
             'driver_nl80211_event.c',
             'driver_nl80211_monitor.c',
             'driver_nl80211_scan.c',
             'driver_hostap.c',
             'netlink.c',
             'linux_ioctl.c',
             'rfkill.c']
# files under hostapd-2.6/src/utils
UTIL_FILES = ['eloop.c',
              'common.c',
              'os_unix.c',
              'ip_addr.c',
              'wpabuf.c',
              'wpa_debug.c',
              'radiotap.c',
              'base64.c',
              'uuid.c']
# files under hostapd-2.6/src/eapol_auth
EAPOL_AUTH_FILES = ['eapol_auth_sm.c', 'eapol_auth_dump.c']
# files under hostapd-2.6/src/radius
RADIUS_FILES = ['radius.c', 'radius_client.c', 'radius_das.c']
# files under hostapd-2.6/src/eap_server
EAP_SERVER_FILES = ['eap_server_md5.c',
                    'eap_server_tls.c',
                    'eap_server_peap.c',
                    'eap_server_ttls.c',
                    'eap_server_mschapv2.c',
                    'eap_server_gtc.c',
                    'eap_server.c',
                    'eap_server_tls_common.c',
                    'eap_server_methods.c',
                    'eap_server_identity.c',
                    'eap_server_wsc.c']
# files under hostapd-2.6/src/eap_common
EAP_COMMON_FILES = ['eap_peap_common.c',
                    'eap_common.c',
                    'chap.c',
                    'eap_wsc_common.c']
# files under hostapd-2.6/src/crypto
CRYPTO_FILES = ['ms_funcs.c',
                'tls_openssl.c',
                'crypto_openssl.c',
                'sha1-prf.c',
                'sha256-prf.c',
                'sha256-kdf.c',
                'sha256-tlsprf.c',
                'random.c']
# files under hostapd-2.6/src/wps
WPS_FILES = ['wps.c',
             'wps_common.c',
             'wps_attr_parse.c',
             'wps_attr_build.c',
             'wps_attr_process.c',
             'wps_dev_attr.c',
             'wps_enrollee.c',
             'wps_registrar.c',
             'wps_upnp.c',
             'wps_upnp_ssdp.c',
             'wps_upnp_event.c',
             'upnp_xml.c',
             'wps_upnp_web.c',
             'http_server.c',
             'http_client.c',
             'httpread.c',
             'wps_upnp_ap.c']
# required hostapd macros
HOSTAPD_MACROS = [('HOSTAPD', '1'),
                  ('CONFIG_NO_VLAN', '1'),
                  ('CONFIG_CTRL_IFACE', '1'),
                  ('CONFIG_CTRL_IFACE_UNIX', '1'),
                  ('EAP_SERVER_MD5', '1'),
                  ('EAP_SERVER_TLS', '1'),
                  ('EAP_SERVER_PEAP', '1'),
                  ('EAP_SERVER_TTLS', '1'),
                  ('EAP_SERVER_MSCHAPV2', '1'),
                  ('EAP_SERVER_WSC', '1'),
                  ('EAP_SERVER_IDENTITY', '1'),
                  ('EAP_SERVER', '1'),
                  ('PKCS12_FUNCS', '1'),
                  ('CONFIG_SHA256', '1'),
                  ('CONFIG_DRIVER_WIRED', '1'),
                  ('CONFIG_LIBNL20', '1'),
                  ('CONFIG_DRIVER_HOSTAP', '1'),
                  ('CONFIG_KARMA_ATTACK', '1'),
                  ('CONFIG_WIFIPHISHER_COMMON', '1'),
                  ('CONFIG_DRIVER_HOSTAP', '1'),
                  ('CONFIG_DRIVER_NL80211', '1'),
                  ('CONFIG_LIBNL32', '1'),
                  ('CONFIG_EAP', '1'),
                  ('CONFIG_ERP', '1'),
                  ('CONFIG_EAP_MD5', '1'),
                  ('CONFIG_EAP_TLS', '1'),
                  ('CONFIG_EAP_MSCHAPV2', '1'),
                  ('CONFIG_EAP_PEAP', '1'),
                  ('CONFIG_EAP_GTC', '1'),
                  ('CONFIG_EAP_TTLS', '1'),
                  ('CONFIG_WPS', '1'),
                  ('CONFIG_WPS_UPNP', '1'),
                  ('CONFIG_PKCS12', '1'),
                  ('CONFIG_IPV6', '1'),
                  ('CONFIG_IEEE80211N', '1'),
                  ('NEED_AP_MLME', '1')]


def get_file_list(directory, file_list):
    """
    Get the paths of required source files under specific directory
    :param directory: Directory of the require files
    :param file_list: Required files
    :type directory: str
    :param file_list: list
    :return: A file path list for the specific directory
    :rtype: list
    """
    return [os.path.join(directory, filename) for filename in file_list]


def get_all_source_files():
    """
    Get all the source files for building the shared library
    :return: A list of source files
    :rtype: list
    """

    ret_files = []
    ret_files.extend(get_file_list(HOSTAPD_MAIN, MAIN_FILES))
    ret_files.extend(get_file_list(HOSTAPD_L2_PACKET, L2_PACKET_FILES))
    ret_files.extend(get_file_list(HOSTAPD_AP, AP_FILES))
    ret_files.extend(get_file_list(HOSTAPD_COMMON, COMMON_FILES))
    ret_files.extend(get_file_list(HOSTAPD_DRVS, DRV_FILES))
    ret_files.extend(get_file_list(HOSTAPD_UTILS, UTIL_FILES))
    ret_files.extend(get_file_list(HOSTAPD_EAPOL_AUTH, EAPOL_AUTH_FILES))
    ret_files.extend(get_file_list(HOSTAPD_RADIUS, RADIUS_FILES))
    ret_files.extend(get_file_list(HOSTAPD_EAP_SERVER, EAP_SERVER_FILES))
    ret_files.extend(get_file_list(HOSTAPD_EAP_COMMON, EAP_COMMON_FILES))
    ret_files.extend(get_file_list(HOSTAPD_CRYPTO, CRYPTO_FILES))
    ret_files.extend(get_file_list(HOSTAPD_WPS, WPS_FILES))
    return ret_files



"""
Module defines the utility functions used in roguehostapd
"""

import contextlib
import io
import os
import sys
import shutil
from textwrap import dedent
import tempfile
import distutils.sysconfig
import distutils.ccompiler
from distutils.core import Extension
from distutils.errors import CompileError, LinkError
import roguehostapd.buildutil.build_files as build_files
import roguehostapd.buildutil.buildexception as buildexception


# code for checking if libnl-dev and libnl-genl-dev exist
LIBNL_CODE = dedent("""
#include <netlink/netlink.h>
#include <netlink/genl/genl.h>
int main(int argc, char* argv[])
{
   struct nl_msg *testmsg;
   testmsg = nlmsg_alloc();
   nlmsg_free(testmsg);
   return 0;
}
""")

# code for checking if openssl library exist
OPENSSL_CODE = dedent("""
#include <openssl/ssl.h>
#include <openssl/err.h>
int main(int argc, char* argv[])
{
    SSL_load_error_strings();
    return 0;
}
""")

LIBNAME_CODE_DICT = {
    "netlink": LIBNL_CODE,
    "openssl": OPENSSL_CODE
}

@contextlib.contextmanager
def nostdout():
    """
    Hide the stdout in the specific context

    :return: None
    :rtype: None
    """
    save_stdout = sys.stdout
    sys.stdout = io.BytesIO()
    yield
    sys.stdout = save_stdout

def check_required_library(libname, libraries=None, include_dir=None):
    """
    Check if the required shared library exists

    :param libname: The name of shared library
    :type libname: str
    :return True if the required shared lib exists else false
    :rtype: bool
    """
    build_success = True
    tmp_dir = tempfile.mkdtemp(prefix='tmp_' + libname + '_')
    bin_file_name = os.path.join(tmp_dir, 'test_' + libname)
    file_name = bin_file_name + '.c'
    with open(file_name, 'w') as filep:
        filep.write(LIBNAME_CODE_DICT[libname])
    compiler = distutils.ccompiler.new_compiler()
    distutils.sysconfig.customize_compiler(compiler)
    try:
        compiler.link_executable(
            compiler.compile([file_name],
                             include_dirs=include_dir),
            bin_file_name,
            libraries=libraries,
        )
    except CompileError:
        build_success = False
    except LinkError:
        build_success = False
    finally:
        shutil.rmtree(tmp_dir)
    if build_success:
        return True
    return False

def get_extension_module():
    """
    Get hostapd extension module
    :return: list of extension for hostapd
    :rtype: list of Extension if build success
    ..note: if the required shared library is missing this function will
    raise the custom exception: SharedLibMissError
    """

    if not check_required_library("netlink", ["nl-3", "nl-genl-3"],
                                  [build_files.LIB_NL3_PATH]):
        raise buildexception.SharedLibMissError('netlink', ['libnl-3-dev',
                                                            'libnl-genl-3-dev'])
    if not check_required_library("openssl", ["ssl"],
                                  [build_files.LIB_SSL_PATH]):
        raise buildexception.SharedLibMissError("openssl", ['libssl-dev'])

    # required shared libraries exist
    ext_module = Extension(build_files.SHARED_LIB_PATH,
                           define_macros=build_files.HOSTAPD_MACROS,
                           libraries=['rt', 'ssl', 'crypto', 'nl-3', 'nl-genl-3'],
                           sources=build_files.get_all_source_files(),
                           include_dirs=[build_files.HOSTAPD_SRC,
                                         build_files.HOSTAPD_UTILS,
                                         build_files.LIB_NL3_PATH])
    return [ext_module]


if __name__ == "__main__":
    check_required_library("netlink", ["nl-3", "nl-genl-3"],
                           [build_files.LIB_NL3_PATH])
    check_required_library("openssl", ["ssl"],
                           [build_files.LIB_SSL_PATH])



#!/usr/bin/python
#
# Example nfcpy to hostapd wrapper for WPS NFC operations
# Copyright (c) 2012-2013, Jouni Malinen <j@w1.fi>
#
# This software may be distributed under the terms of the BSD license.
# See README for more details.

import os
import sys
import time
import argparse

import nfc
import nfc.ndef
import nfc.llcp
import nfc.handover

import logging

import wpaspy

wpas_ctrl = '/var/run/hostapd'
continue_loop = True
summary_file = None
success_file = None

def summary(txt):
    print txt
    if summary_file:
        with open(summary_file, 'a') as f:
            f.write(txt + "\n")

def success_report(txt):
    summary(txt)
    if success_file:
        with open(success_file, 'a') as f:
            f.write(txt + "\n")

def wpas_connect():
    ifaces = []
    if os.path.isdir(wpas_ctrl):
        try:
            ifaces = [os.path.join(wpas_ctrl, i) for i in os.listdir(wpas_ctrl)]
        except OSError, error:
            print "Could not find hostapd: ", error
            return None

    if len(ifaces) < 1:
        print "No hostapd control interface found"
        return None

    for ctrl in ifaces:
        try:
            wpas = wpaspy.Ctrl(ctrl)
            return wpas
        except Exception, e:
            pass
    return None


def wpas_tag_read(message):
    wpas = wpas_connect()
    if (wpas == None):
        return False
    if "FAIL" in wpas.request("WPS_NFC_TAG_READ " + str(message).encode("hex")):
        return False
    return True


def wpas_get_config_token():
    wpas = wpas_connect()
    if (wpas == None):
        return None
    ret = wpas.request("WPS_NFC_CONFIG_TOKEN NDEF")
    if "FAIL" in ret:
        return None
    return ret.rstrip().decode("hex")


def wpas_get_password_token():
    wpas = wpas_connect()
    if (wpas == None):
        return None
    ret = wpas.request("WPS_NFC_TOKEN NDEF")
    if "FAIL" in ret:
        return None
    return ret.rstrip().decode("hex")


def wpas_get_handover_sel():
    wpas = wpas_connect()
    if (wpas == None):
        return None
    ret = wpas.request("NFC_GET_HANDOVER_SEL NDEF WPS-CR")
    if "FAIL" in ret:
        return None
    return ret.rstrip().decode("hex")


def wpas_report_handover(req, sel):
    wpas = wpas_connect()
    if (wpas == None):
        return None
    return wpas.request("NFC_REPORT_HANDOVER RESP WPS " +
                        str(req).encode("hex") + " " +
                        str(sel).encode("hex"))


class HandoverServer(nfc.handover.HandoverServer):
    def __init__(self, llc):
        super(HandoverServer, self).__init__(llc)
        self.ho_server_processing = False
        self.success = False

    # override to avoid parser error in request/response.pretty() in nfcpy
    # due to new WSC handover format
    def _process_request(self, request):
        summary("received handover request {}".format(request.type))
        response = nfc.ndef.Message("\xd1\x02\x01Hs\x12")
        if not request.type == 'urn:nfc:wkt:Hr':
            summary("not a handover request")
        else:
            try:
                request = nfc.ndef.HandoverRequestMessage(request)
            except nfc.ndef.DecodeError as e:
                summary("error decoding 'Hr' message: {}".format(e))
            else:
                response = self.process_request(request)
        summary("send handover response {}".format(response.type))
        return response

    def process_request(self, request):
        summary("HandoverServer - request received")
        try:
            print "Parsed handover request: " + request.pretty()
        except Exception, e:
            print e
        print str(request).encode("hex")

        sel = nfc.ndef.HandoverSelectMessage(version="1.2")

        for carrier in request.carriers:
            print "Remote carrier type: " + carrier.type
            if carrier.type == "application/vnd.wfa.wsc":
                summary("WPS carrier type match - add WPS carrier record")
                data = wpas_get_handover_sel()
                if data is None:
                    summary("Could not get handover select carrier record from hostapd")
                    continue
                print "Handover select carrier record from hostapd:"
                print data.encode("hex")
                if "OK" in wpas_report_handover(carrier.record, data):
                    success_report("Handover reported successfully")
                else:
                    summary("Handover report rejected")

                message = nfc.ndef.Message(data);
                sel.add_carrier(message[0], "active", message[1:])

        print "Handover select:"
        try:
            print sel.pretty()
        except Exception, e:
            print e
        print str(sel).encode("hex")

        summary("Sending handover select")
        self.success = True
        return sel


def wps_tag_read(tag):
    success = False
    if len(tag.ndef.message):
        for record in tag.ndef.message:
            print "record type " + record.type
            if record.type == "application/vnd.wfa.wsc":
                summary("WPS tag - send to hostapd")
                success = wpas_tag_read(tag.ndef.message)
                break
    else:
        summary("Empty tag")

    if success:
        success_report("Tag read succeeded")

    return success


def rdwr_connected_write(tag):
    summary("Tag found - writing - " + str(tag))
    global write_data
    tag.ndef.message = str(write_data)
    success_report("Tag write succeeded")
    print "Done - remove tag"
    global only_one
    if only_one:
        global continue_loop
        continue_loop = False
    global write_wait_remove
    while write_wait_remove and tag.is_present:
        time.sleep(0.1)

def wps_write_config_tag(clf, wait_remove=True):
    summary("Write WPS config token")
    global write_data, write_wait_remove
    write_wait_remove = wait_remove
    write_data = wpas_get_config_token()
    if write_data == None:
        summary("Could not get WPS config token from hostapd")
        return

    print "Touch an NFC tag"
    clf.connect(rdwr={'on-connect': rdwr_connected_write})


def wps_write_password_tag(clf, wait_remove=True):
    summary("Write WPS password token")
    global write_data, write_wait_remove
    write_wait_remove = wait_remove
    write_data = wpas_get_password_token()
    if write_data == None:
        summary("Could not get WPS password token from hostapd")
        return

    print "Touch an NFC tag"
    clf.connect(rdwr={'on-connect': rdwr_connected_write})


def rdwr_connected(tag):
    global only_one, no_wait
    summary("Tag connected: " + str(tag))

    if tag.ndef:
        print "NDEF tag: " + tag.type
        try:
            print tag.ndef.message.pretty()
        except Exception, e:
            print e
        success = wps_tag_read(tag)
        if only_one and success:
            global continue_loop
            continue_loop = False
    else:
        summary("Not an NDEF tag - remove tag")
        return True

    return not no_wait


def llcp_startup(clf, llc):
    print "Start LLCP server"
    global srv
    srv = HandoverServer(llc)
    return llc

def llcp_connected(llc):
    print "P2P LLCP connected"
    global wait_connection
    wait_connection = False
    global srv
    srv.start()
    return True


def main():
    clf = nfc.ContactlessFrontend()

    parser = argparse.ArgumentParser(description='nfcpy to hostapd integration for WPS NFC operations')
    parser.add_argument('-d', const=logging.DEBUG, default=logging.INFO,
                        action='store_const', dest='loglevel',
                        help='verbose debug output')
    parser.add_argument('-q', const=logging.WARNING, action='store_const',
                        dest='loglevel', help='be quiet')
    parser.add_argument('--only-one', '-1', action='store_true',
                        help='run only one operation and exit')
    parser.add_argument('--no-wait', action='store_true',
                        help='do not wait for tag to be removed before exiting')
    parser.add_argument('--summary',
                        help='summary file for writing status updates')
    parser.add_argument('--success',
                        help='success file for writing success update')
    parser.add_argument('command', choices=['write-config',
                                            'write-password'],
                        nargs='?')
    args = parser.parse_args()

    global only_one
    only_one = args.only_one

    global no_wait
    no_wait = args.no_wait

    if args.summary:
        global summary_file
        summary_file = args.summary

    if args.success:
        global success_file
        success_file = args.success

    logging.basicConfig(level=args.loglevel)

    try:
        if not clf.open("usb"):
            print "Could not open connection with an NFC device"
            raise SystemExit

        if args.command == "write-config":
            wps_write_config_tag(clf, wait_remove=not args.no_wait)
            raise SystemExit

        if args.command == "write-password":
            wps_write_password_tag(clf, wait_remove=not args.no_wait)
            raise SystemExit

        global continue_loop
        while continue_loop:
            print "Waiting for a tag or peer to be touched"
            wait_connection = True
            try:
                if not clf.connect(rdwr={'on-connect': rdwr_connected},
                                   llcp={'on-startup': llcp_startup,
                                         'on-connect': llcp_connected}):
                    break
            except Exception, e:
                print "clf.connect failed"

            global srv
            if only_one and srv and srv.success:
                raise SystemExit

    except KeyboardInterrupt:
        raise SystemExit
    finally:
        clf.close()

    raise SystemExit

if __name__ == '__main__':
    main()



"""
Module for setup hostapd shared library
"""

import shutil
from distutils.core import setup
import roguehostapd.hostapd_constants as constants
import roguehostapd.buildutil.buildcommon as buildcommon
import roguehostapd.buildutil.buildexception as buildexception

# define project information
NAME = 'roguehostapd'
PACKAGES = ['roguehostapd']
PACKAGE_DIR = {'roguehostapd': 'roguehostapd'}
PACKAGE_DATA = {'roguehostapd': ['hostapd-2_6/hostapd/hostapd.conf']}
VERSION = '1.1.9'
DESCRIPTION = 'Hostapd wrapper for hostapd'
URL = 'https://github.com/wifiphisher/roguehostapd'
AUTHOR = 'Anakin'

try:
    EXT_MODULE = buildcommon.get_extension_module()
    # hide the stdout of building process
    with buildcommon.nostdout():
        setup(
            name=NAME,
            packages=PACKAGES,
            package_dir=PACKAGE_DIR,
            package_data=PACKAGE_DATA,
            version=VERSION,
            description=DESCRIPTION,
            url=URL,
            author=AUTHOR,
            ext_modules=EXT_MODULE
        )
except buildexception.SharedLibMissError as exobj:
    print ("[" + constants.RED + "!" + constants.WHITE + "] " +
           ("The development package for " + exobj.libname +
            " is missing. Please download it and restart the compilation."
            " Now if you want, you can provide the exact command for Debian-based systems."
            ' For example, "if you are on Debian-based system: \'apt-get install{}\'."'.format(
                "".join(" " + package for package in exobj.packages))))
    with buildcommon.nostdout():
        setup(
            name=NAME,
            packages=PACKAGES,
            package_dir=PACKAGE_DIR,
            package_data=PACKAGE_DATA,
            version=VERSION,
            description=DESCRIPTION,
            url=URL,
            author=AUTHOR,
        )
finally:
    shutil.rmtree('tmp')



"""
Module defines the custom exceptions for building hostapd
"""


class SharedLibMissError(Exception):
    """
    Define the Netlink shared library missing exception
    """
    def __init__(self, libname, packages):
        """
        Initialize the NetlinkMissError object
        param self: A SharedLibMissError object
        param libname: Required library name
        param packages: The packages required by the lib
        type self: SharedLibMissError
        type libname: str
        type packages: list
        return: None
        rtype: None
        """
        super(SharedLibMissError, self).__init__(libname, packages)
        self.libname = libname
        self.packages = packages
